@@ -1,11 +1,11 @@
 ;;; kind-icon.el --- Completion kind icons  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021  Free Software Foundation, Inc.
+;; Copyright (C) 2021-2022  Free Software Foundation, Inc.
 
 ;; Author: J.D. Smith <jdtsmith@gmail.com>
-;; Homepage: https://github.com/jdtsmith/kind-icon
+;; URL: https://github.com/jdtsmith/kind-icon
 ;; Package-Requires: ((emacs "27.1") svg-lib)
-;; Package-Version: 0.1.4
+;; Version: 0.1.6
 ;; Keywords: completion
 
 ;; kind-icon is free software: you can redistribute it
@@ -58,15 +58,16 @@
 (eval-when-compile
   (require 'subr-x))
 (require 'svg-lib nil 'noerror)
+(require 'color)
 
-(defvar kind-icon--cache nil
+(defvar kind-icon--cache [nil nil]
   "The cache of styled and padded label (text or icon).  
-An alist.")
+An vector of two alist for non-terminal and terminal.")
 
 (defun kind-icon-reset-cache ()
   "Remove all cached icons from `kind-icon-mapping'."
   (interactive)
-  (setq kind-icon--cache nil))
+  (setq kind-icon--cache (make-vector 2 nil)))
 
 (defun kind-icon--set-default-clear-cache (&rest args)
   (kind-icon-reset-cache)
@@ -204,8 +205,6 @@ Uses svg-lib, guarding against non-availability or network errors."
   "Return a fractional blend between two colors RGB1 and RGB2.
 Each is a 3 element list.  The fractional blend point is the
 float FRAC."
-  ;; FIXME: `color-rgb-to-hex' won't be (auto)loaded if `svg-lib' is
-  ;; not installed.
   (apply #'color-rgb-to-hex
 	 (cl-mapcar (lambda (a b)
 		      (+ (* a frac) (* b (- 1.0 frac))))
@@ -216,6 +215,31 @@ float FRAC."
    (plist-get completion-extra-properties (intern (format ":%s" type-name)))
    (cdr (assq (intern type-name) metadata))))
 
+(defconst kind-icon--unknown
+  (propertize "???" 'face '(:weight bold :background "red")))
+
+(defun kind-icon-preview-all ()
+  "Preview all kind icons.
+In the process, svg-lib also downloads and caches them."
+  (interactive)
+  (with-current-buffer (get-buffer-create "*kind-icon-preview*")
+    (font-lock-mode 0)
+    (view-buffer-other-window (current-buffer))
+    (let ((inhibit-read-only t))
+      (erase-buffer)
+      (insert "kind-icon badges\n\ntxt icn\tkind\n")
+      (mapc (lambda (k)
+	      (apply 'insert
+		     `(,(mapconcat
+			 (lambda (v)
+			   (kind-icon-reset-cache)
+			   (let ((kind-icon-use-icons v))
+			     (kind-icon-formatted k)))
+			 '(nil t) " ")
+		       "\t" ,(symbol-name k) "\n")))
+	    (mapcar 'car kind-icon-mapping)))
+    (help-mode)))
+
 (defun kind-icon-formatted (kind)
   "Return a formatted kind badge, either icon or text abbreviation.
 Caches this badge in `kind-icon--cache', and returns the cached
@@ -230,57 +254,63 @@ frame background color) and the foreground.  If
 `kind-icon-blend-background' is nil, the background is taken from
 the :face's background, `kind-icon-default-face', or the frame
 background-color."
-  (or (alist-get kind kind-icon--cache)
-      (if-let ((map (assq kind kind-icon-mapping))
-	       (plist (cddr map)))
-	  (let* ((kind-face (plist-get plist :face))
-		 (col (if kind-face
-			  (face-attribute kind-face :foreground nil t)
-			(if kind-icon-default-face
-			    (face-attribute kind-icon-default-face :foreground nil t)
-			  (frame-parameter nil 'foreground-color))))
-		 (kind-face-bg (and kind-face
-				    (face-attribute kind-face :background nil t)))
-		 (default-bg (if kind-icon-default-face
-				 (face-attribute kind-icon-default-face :background nil t)
-			       (frame-parameter nil 'background-color)))
-		 (bg-col (if kind-icon-blend-background
-			     (kind-icon--rgb-blend
-			      (color-name-to-rgb col)
-			      (color-name-to-rgb default-bg)
-			      kind-icon-blend-frac)
-			   (if (and kind-face-bg (not (eq kind-face-bg 'unspecified)))
-			       kind-face-bg
-			     default-bg)))
-		 (dfw (default-font-width))
-		 (half (/ dfw 2))
-		 (pad-right (propertize " " 'display `(space :width (,half))))
-		 (pad-left (propertize " " 'display `(space :width (,(- dfw half)))))
-		 (disp (if-let ((kind-icon-use-icons)
-				(icon-name (plist-get plist :icon))
-				(icon (kind-icon--get-icon-safe icon-name col bg-col)))
-			   ;; icon: always 2x1, half-space on each side
-			   (propertize ; pretend it's one char to allow padding
-			    (concat pad-left (propertize "*" 'display icon) pad-right)
-			    'face `(:background ,bg-col))
-			 ;; text, 1 or 2 chars, centered with full or half space on each side
-			 (let* ((txt (truncate-string-to-width (cadr map) 2))
-				(len (length txt))
-				(txt-disp (if (eq len 2)
-					      (concat pad-left
-						      (propertize "_" 'display txt)
-						      pad-right)
-					    (concat " " txt " "))))
-			   (propertize txt-disp 'face
-				       `(:weight bold :foreground ,col :background ,bg-col))))))
-	    (if disp
-		(setf (alist-get kind kind-icon--cache) disp)
-	      (propertize (concat pad-left "??" pad-right) 'face font-lock-warning-face)))
-	kind-icon--unknown)))
-
-(defconst kind-icon--unknown
-  (propertize "???" 'face '(:weight bold :background "red")))
+  (let* ((dfw (default-font-width))
+	 (terminal (eq dfw 1))
+	 (slot (if terminal 1 0)))
+    (or (alist-get kind (aref kind-icon--cache slot))
+	(if-let ((map (assq kind kind-icon-mapping))
+		 (plist (cddr map)))
+	    (let* ((kind-face (plist-get plist :face))
+		   (col (if kind-face
+			    (face-attribute kind-face :foreground nil t)
+			  (if kind-icon-default-face
+			      (face-attribute kind-icon-default-face :foreground nil t)
+			    (frame-parameter nil 'foreground-color))))
+		   (kind-face-bg (and kind-face
+				      (face-attribute kind-face :background nil t)))
+		   (default-bg (if kind-icon-default-face
+				   (face-attribute kind-icon-default-face :background nil t)
+				 (frame-parameter nil 'background-color)))
+		   (bg-col (if (and kind-icon-blend-background (not terminal))
+			       (kind-icon--rgb-blend
+				(color-name-to-rgb col)
+				(color-name-to-rgb default-bg)
+				kind-icon-blend-frac)
+			     (if (and kind-face-bg (not (eq kind-face-bg 'unspecified)))
+				 kind-face-bg
+			       default-bg)))
+		   (half (/ dfw 2))
+		   (face-spec `(:weight bold :foreground ,col :background ,bg-col))
+		   (pad-right (propertize " " 'display `(space :width (,half))
+					  'face face-spec))
+		   (pad-left (propertize " " 'display `(space :width (,(- dfw half)))
+					 'face face-spec))
+		   (disp (if-let ((kind-icon-use-icons)
+				  ((not terminal))
+				  (icon-name (plist-get plist :icon))
+				  (icon (kind-icon--get-icon-safe icon-name col bg-col)))
+			     ;; icon: always 2x1, half-space on each side
+			     (propertize ; pretend it's one char to allow padding
+			      (concat pad-left
+				      (propertize "*" 'display icon 'face `(:background ,bg-col))
+				      pad-right))
+			   ;; text, 1 or 2 chars, centered with full or half space on each side
+			   (let* ((txt (truncate-string-to-width (cadr map) 2))
+				  (len (length txt)))
+			     (if (eq len 2)
+				 (if terminal ; no half spaces on terminal
+				     (propertize (concat txt " ") 'face face-spec)
+				   (concat pad-left
+					   (propertize "_" 'display
+						       (propertize txt 'face face-spec))
+					   pad-right))
+			       (propertize (concat " " txt " ") 'face face-spec))))))
+	      (if disp
+		  (setf (alist-get kind (aref kind-icon--cache slot)) disp)
+		(propertize (concat pad-left "??" pad-right) 'face font-lock-warning-face)))
+	  kind-icon--unknown))))
 
+;;;###autoload
 (defun kind-icon-margin-formatter (metadata)
   "Return a margin-formatter function which produces kind icons.
 METADATA is the completion metadata supplied by the caller (see
@@ -296,7 +326,7 @@ function to the relevant margin-formatters list."
   "Create and return a custom kind-icon affixation function.
 The company-kind function should be passed in as KIND-FUNC and
 any annotation-function as ANN-FUNC.  The returned function
-supplies a candiate kind badge -- abbreviated text key or icon --
+supplies a candidate kind badge -- abbreviated text key or icon --
 as an affixation prefix.  ANN-FUNC, if non-nil, will be called
 and its result used as the affixation suffix, first setting the
 `completions-annotations' face on it."
@@ -313,26 +343,27 @@ and its result used as the affixation suffix, first setting the
 			  suffix))))
 	      candidates)))
 
+;;;###autoload
 (defun kind-icon-enhance-completion (completion-function)
-    "A wrapper for completion-in-region-functions.
+  "A wrapper for completion-in-region-functions.
 This wrapper sets a custom affixation-function which places an
 icon in the prefix slot. Use it like:
 
   (setq completion-in-region-function 
      (kind-icon-enhance-completion 
        completion-in-region-function))"
-    (lambda (start end table &optional pred)
-      (let* ((str (buffer-substring start (point)))
-	     (metadata (completion-metadata str table pred))
-	     (kind-func (kind-icon--metadata-get metadata "company-kind"))
-	     (ann-func (kind-icon--metadata-get metadata "annotation-function"))
-	     (aff-func (kind-icon--metadata-get metadata "affixation-function")))
-	(when (and kind-func (not aff-func)) ;; add a custom affixation function
-	  (kind-icon-reset-cache)
-	  (setq completion-extra-properties
-		(plist-put completion-extra-properties :affixation-function
-			   (kind-icon--affixation-function kind-func ann-func)))))
-      (funcall completion-function start end table pred)))
+  (lambda (start end table &optional pred)
+    (let* ((str (buffer-substring start (point)))
+	   (metadata (completion-metadata str table pred))
+	   (kind-func (kind-icon--metadata-get metadata "company-kind"))
+	   (ann-func (kind-icon--metadata-get metadata "annotation-function"))
+	   (aff-func (kind-icon--metadata-get metadata "affixation-function")))
+      (when (and kind-func (not aff-func)) ;; add a custom affixation function
+	(kind-icon-reset-cache)
+	(setq completion-extra-properties
+	      (plist-put completion-extra-properties :affixation-function
+			 (kind-icon--affixation-function kind-func ann-func)))))
+    (funcall completion-function start end table pred)))
 
 (provide 'kind-icon)
 ;;; kind-icon.el ends here.
